@@ -3,6 +3,7 @@ import jwt
 from flask import Flask, jsonify, request
 from flask.json import JSONEncoder
 from werkzeug.routing import BaseConverter
+from inspect import signature
 from .page import Page
 from .element import Element
 
@@ -10,6 +11,8 @@ class CallbackRegistryType:
     uuid_callback_map = {}
     callback_uuid_map = {}
     def uuid_for_callback(self, callback):
+        if callback is None:
+            return None
         if callback in self.callback_uuid_map:
             return self.callback_uuid_map[callback]
         else:
@@ -19,7 +22,9 @@ class CallbackRegistryType:
             return cb_uuid
     def make_callback(self, uuid, args):
         if uuid in self.uuid_callback_map:
-            return self.uuid_callback_map[uuid](*args)
+            method = self.uuid_callback_map[uuid]
+            param_length = len(signature(method).parameters)
+            return method(*args[:param_length])
         else:
             # TODO: Return an error to the frontend
             return None
@@ -34,7 +39,6 @@ class ElementJSONEncoder(JSONEncoder):
     def default(self, obj):
         if isinstance(obj, Element):
             return obj.as_dict()
-        return super(MyJSONEncoder, self).default(obj)
 
 class MenuItem(Element):
     """Represents a menu item
@@ -76,13 +80,13 @@ class LoggedInUser(Element):
         avatar: the avatar of the user
         user_info: info for future use, accessible by app.current_user()['user_info']
     """
-    def __init__(self, display_name='', auth=['user'], avatar=DEFAULT_AVATAR, user_info=None):
+    def __init__(self, display_name='', auth=['user'], avatar=DEFAULT_AVATAR, user_info=None, redirect_to=None):
         token = jwt.encode({
             "display_name": display_name,
             "auth": auth,
             "user_info": user_info
         }, AdminApp.SECRET, algorithm='HS256')
-        super().__init__('LoginAndNavigateTo', status='ok', display_name=display_name, avatar=avatar, token=token.decode('utf-8'))
+        super().__init__('LoginAndNavigateTo', status='ok', display_name=display_name, avatar=avatar, redirect_to=redirect_to, token=token.decode('utf-8'))
 
 class LoginFailed(Element):
     """Returned by login handler, represent a failed login attempt
@@ -101,6 +105,8 @@ class ErrorResponse(Element):
 class AdminApp:
     """Create an AdminUI App"""
     SECRET = "admin ui super &*#*$ secret"
+    app_title = 'Admin UI App'
+    app_logo = None
 
     def __init__(self):
         self.app = Flask(__name__, static_url_path='/')
@@ -212,18 +218,26 @@ class AdminApp:
             'menu': [x.as_dict() for x in self.menu if x.has_auth(token['auth'])]
         })
 
+    def serve_settings(self):
+        """Serve settings like logo and title"""
+        return jsonify({'title':self.app_title, 'logo':self.app_logo})
+
     def serve_root(self, path=''):
         """!!! Private method, don't call. Serve the index.html"""
         return self.app.send_static_file('index.html')
 
     def run(self):
         """run the AdminUI App"""
-        # self.app.route('/api/page_layout/<url>')(self.serve_page)
+        self.prepare().run()
+
+    def prepare(self):
+        """do the prepare work and return the flask app"""
         self.app.route('/api/page_layout/<path:url>/')(self.serve_page)
         self.app.route('/api/page_layout/')(self.serve_page)
         self.app.route('/api/main_menu')(self.serve_menu)
+        self.app.route('/api/app_settings')(self.serve_settings)
         self.app.route('/api/login', methods=['POST'])(self.handle_login_action)
         self.app.route('/api/page_action', methods=['POST'])(self.handle_page_action)
         self.app.route('/')(self.serve_root)
         self.app.route('/<purePath:path>/')(self.serve_root)
-        self.app.run()
\ No newline at end of file
+        return self.app
\ No newline at end of file
